"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vitest_1 = require("vitest");
const some_1 = require("./some");
const none_1 = require("./none");
const success_1 = require("../result/success");
(0, vitest_1.describe)("Some Tests", () => {
    (0, vitest_1.test)("getで値を取り出せる", () => {
        const option = (0, some_1.some)(10);
        (0, vitest_1.expect)(option.get).toBe(10);
    });
    (0, vitest_1.test)("valueで値を取り出せる", () => {
        const option = (0, some_1.some)(10);
        (0, vitest_1.expect)(option.value).toBe(10);
    });
    (0, vitest_1.test)("getOrElseで値を取り出せる", () => {
        const option = (0, some_1.some)("hello");
        (0, vitest_1.expect)(option.getOrElse("world")).toBe("hello");
    });
    (0, vitest_1.test)("orElseでインスタンスがそのまま出てくる", () => {
        const option = (0, some_1.some)("hello");
        (0, vitest_1.expect)(option.orElse((0, some_1.some)("world"))).toEqual(option);
    });
    (0, vitest_1.test)("isDefinedはtrue", () => {
        const option = (0, some_1.some)("hello");
        (0, vitest_1.expect)(option.isDefined).toBeTruthy();
    });
    (0, vitest_1.test)("isEmptyはfalse", () => {
        const option = (0, some_1.some)("hello");
        (0, vitest_1.expect)(option.isEmpty).toBeFalsy();
    });
    (0, vitest_1.test)("mapで値を変換できる", () => {
        const option = (0, some_1.some)(10);
        (0, vitest_1.expect)(option.map((value) => value * 2)).toEqual((0, some_1.some)(20));
    });
    (0, vitest_1.test)("flatMapで値を変換できる", () => {
        const option = (0, some_1.some)(10);
        (0, vitest_1.expect)(option.flatMap((value) => (0, some_1.some)(value * 2))).toEqual((0, some_1.some)(20));
    });
    (0, vitest_1.test)("flatMapでnoneが帰ってきたらnoneを返す", () => {
        const option = (0, some_1.some)(10);
        (0, vitest_1.expect)(option.flatMap((_) => (0, none_1.none)())).toEqual((0, none_1.none)());
    });
    (0, vitest_1.test)("同じ値同士ならequalsでtrueを返す", () => {
        const option1 = (0, some_1.some)(10);
        const option2 = (0, some_1.some)(10);
        (0, vitest_1.expect)(option1.equals(option2)).toBeTruthy();
    });
    (0, vitest_1.test)("異なる値同士ならequalsでfalseを返す", () => {
        const option1 = (0, some_1.some)(10);
        const option2 = (0, some_1.some)(20);
        (0, vitest_1.expect)(option1.equals(option2)).toBeFalsy();
    });
    (0, vitest_1.test)("引数に関数が与えられた場合はsome同士ならば関数の戻り値がtrueならばtrueを返す", () => {
        const option1 = (0, some_1.some)({ num: 3 });
        const option2 = (0, some_1.some)({ num: 3 });
        (0, vitest_1.expect)(option1.equals(option2, (a, b) => a.num === b.num)).toBeTruthy();
    });
    (0, vitest_1.test)("引数に関数が与えられた場合はsome同士ならば関数の戻り値がfalseならばfalseを返す", () => {
        const option1 = (0, some_1.some)({ num: 3 });
        const option2 = (0, some_1.some)({ num: 3 });
        (0, vitest_1.expect)(option1.equals(option2, () => false)).toBeFalsy();
    });
    (0, vitest_1.test)("noneと比較するとfalseを返す", () => {
        const option1 = (0, some_1.some)(10);
        const option2 = (0, none_1.none)();
        (0, vitest_1.expect)(option1.equals(option2)).toBeFalsy();
    });
    (0, vitest_1.test)("foldで第一引数の戻り値を返す", () => {
        const option = (0, some_1.some)(10);
        (0, vitest_1.expect)(option.fold(() => 0, (value) => value * 2)).toBe(20);
    });
    (0, vitest_1.test)("unwrapで値を取り出せる", () => {
        const option = (0, some_1.some)(10);
        (0, vitest_1.expect)(option.unwrap(new Error("Error"))).toBe(10);
    });
    (0, vitest_1.test)("toResultでsuccessに変換する", () => {
        const option = (0, some_1.some)(10);
        (0, vitest_1.expect)(option.toResult(new Error("error"))).toEqual((0, success_1.success)(10));
    });
});
//# sourceMappingURL=some.test.js.map